"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.any = anyToString;
exports.array = arrayToString;
exports.bigint = bigintToString;
exports.boolean = booleanToString;
exports.eq = eqToString;
exports.integer = integerToString;
exports.intersect = intersectToString;
exports.never = neverToString;
exports.null = nullToString;
exports.number = numberToString;
exports.object = objectToString;
exports.optional = optionalToString;
exports.record = recordToString;
exports.string = stringToString;
exports.symbol = symbolToString;
exports.toType = toType;
exports.tuple = tupleToString;
exports.undefined = undefinedToString;
exports.union = unionToString;
exports.unknown = unknownToString;
exports.void = voidToString;
var _registry = /*#__PURE__*/require("@traversable/registry");
const Symbol_optional = _registry.symbol.optional;
/** @internal */
const isArray = globalThis.Array.isArray;
/** @internal */
const hasToType = x => !!x && typeof x === 'function' && 'toType' in x && typeof x.toType === 'function';
/** @internal */
const isOptional = u => !!u && typeof u === 'function' && Symbol_optional in u && typeof u[Symbol_optional] === 'number';
/** @internal */
const isShowable = u => u == null || typeof u === 'boolean' || typeof u === 'number' || typeof u === 'bigint' || typeof u === 'string';
/** @internal */
const stringify = u => typeof u === 'string' ? `'${u}'` : isShowable(u) ? globalThis.String(u) : 'string';
function toType(x) {
  return hasToType(x) ? x.toType() : 'unknown';
}
toType.never = 'never';
toType.unknown = 'unknown';
toType.any = 'any';
toType.void = 'void';
toType.undefined = 'undefined';
toType.null = 'null';
toType.symbol = 'symbol';
toType.boolean = 'boolean';
toType.integer = 'number';
toType.number = 'number';
toType.bigint = 'bigint';
toType.string = 'string';
toType.optional = x => `(${toType(x)} | undefined)`;
toType.array = x => '(' + toType(x) + ')[]';
toType.record = x => `Record<string, ${toType(x)}>`;
toType.eq = x => isShowable(typeof x) ? stringify(x) : typeof x === 'symbol' ? 'symbol' : 'string';
toType.union = xs => isArray(xs) ? xs.length === 0 ? 'never' : `(${xs.map(toType).join(' | ')})` : 'unknown';
toType.intersect = xs => isArray(xs) ? xs.length === 0 ? 'unknown' : `(${xs.map(toType).join(' & ')})` : 'unknown';
toType.tuple = xs => isArray(xs) ? `[${xs.map(x => isOptional(x) ? `_?: ${toType(x)}` : toType(x)).join(', ')}]` : 'unknown[]';
toType.object = xs => {
  if (!!xs && typeof xs === 'object') {
    const entries = Object.entries(xs);
    if (entries.length === 0) return '{}';else return `{ ${entries.map(([k, x]) => `'${k}${isOptional(x) ? "'?" : "'"}: ${toType(x)}`).join(', ')} }`;
  } else return '{ [x: string]: unknown }';
};
function neverToString() {
  return toType.never;
}
function unknownToString() {
  return toType.unknown;
}
function anyToString() {
  return toType.any;
}
function voidToString() {
  return toType.void;
}
function undefinedToString() {
  return toType.undefined;
}
function nullToString() {
  return toType.null;
}
function symbolToString() {
  return toType.symbol;
}
function booleanToString() {
  return toType.boolean;
}
function integerToString() {
  return toType.integer;
}
function bigintToString() {
  return toType.bigint;
}
function numberToString() {
  return toType.number;
}
function stringToString() {
  return toType.string;
}
function eqToString() {
  return toType.eq(this.def);
}
function arrayToString() {
  return toType.array(this.def);
}
function optionalToString() {
  return toType.optional(this.def);
}
function recordToString() {
  return toType.record(this.def);
}
function unionToString() {
  return toType.union(this.def);
}
function intersectToString() {
  return toType.intersect(this.def);
}
function tupleToString() {
  return toType.tuple(this.def);
}
function objectToString() {
  return toType.object(this.def);
}
//# sourceMappingURL=toType.js.map