import { symbol } from '@traversable/registry';
export { neverToString as never, unknownToString as unknown, anyToString as any, voidToString as void, nullToString as null, undefinedToString as undefined, symbolToString as symbol, booleanToString as boolean, integerToString as integer, bigintToString as bigint, numberToString as number, stringToString as string, eqToString as eq, arrayToString as array, recordToString as record, unionToString as union, intersectToString as intersect, optionalToString as optional, tupleToString as tuple, objectToString as object };
const Symbol_optional = symbol.optional;
/** @internal */
const isArray = globalThis.Array.isArray;
/** @internal */
const hasToType = x => !!x && typeof x === 'function' && 'toType' in x && typeof x.toType === 'function';
/** @internal */
const isOptional = u => !!u && typeof u === 'function' && Symbol_optional in u && typeof u[Symbol_optional] === 'number';
/** @internal */
const isShowable = u => u == null || typeof u === 'boolean' || typeof u === 'number' || typeof u === 'bigint' || typeof u === 'string';
/** @internal */
const stringify = u => typeof u === 'string' ? `'${u}'` : isShowable(u) ? globalThis.String(u) : 'string';
export function toType(x) {
  return hasToType(x) ? x.toType() : 'unknown';
}
toType.never = 'never';
toType.unknown = 'unknown';
toType.any = 'any';
toType.void = 'void';
toType.undefined = 'undefined';
toType.null = 'null';
toType.symbol = 'symbol';
toType.boolean = 'boolean';
toType.integer = 'number';
toType.number = 'number';
toType.bigint = 'bigint';
toType.string = 'string';
toType.optional = x => `(${toType(x)} | undefined)`;
toType.array = x => '(' + toType(x) + ')[]';
toType.record = x => `Record<string, ${toType(x)}>`;
toType.eq = x => isShowable(typeof x) ? stringify(x) : typeof x === 'symbol' ? 'symbol' : 'string';
toType.union = xs => isArray(xs) ? xs.length === 0 ? 'never' : `(${xs.map(toType).join(' | ')})` : 'unknown';
toType.intersect = xs => isArray(xs) ? xs.length === 0 ? 'unknown' : `(${xs.map(toType).join(' & ')})` : 'unknown';
toType.tuple = xs => isArray(xs) ? `[${xs.map(x => isOptional(x) ? `_?: ${toType(x)}` : toType(x)).join(', ')}]` : 'unknown[]';
toType.object = xs => {
  if (!!xs && typeof xs === 'object') {
    const entries = Object.entries(xs);
    if (entries.length === 0) return '{}';else return `{ ${entries.map(([k, x]) => `'${k}${isOptional(x) ? "'?" : "'"}: ${toType(x)}`).join(', ')} }`;
  } else return '{ [x: string]: unknown }';
};
function neverToString() {
  return toType.never;
}
function unknownToString() {
  return toType.unknown;
}
function anyToString() {
  return toType.any;
}
function voidToString() {
  return toType.void;
}
function undefinedToString() {
  return toType.undefined;
}
function nullToString() {
  return toType.null;
}
function symbolToString() {
  return toType.symbol;
}
function booleanToString() {
  return toType.boolean;
}
function integerToString() {
  return toType.integer;
}
function bigintToString() {
  return toType.bigint;
}
function numberToString() {
  return toType.number;
}
function stringToString() {
  return toType.string;
}
function eqToString() {
  return toType.eq(this.def);
}
function arrayToString() {
  return toType.array(this.def);
}
function optionalToString() {
  return toType.optional(this.def);
}
function recordToString() {
  return toType.record(this.def);
}
function unionToString() {
  return toType.union(this.def);
}
function intersectToString() {
  return toType.intersect(this.def);
}
function tupleToString() {
  return toType.tuple(this.def);
}
function objectToString() {
  return toType.object(this.def);
}
//# sourceMappingURL=toType.js.map